@@ -2,8 +2,8 @@
  * ContextMenu plugin script
  */
 
-rcmail.contextmenu_command_handlers = new Object();
-rcmail.contextmenu_disable_multi = new Array('#reply','#reply-all','#reply-list','#forward','#forward-attachment','#print','#edit','#viewsource','#download','#open','#edit');
+rcube_webmail.prototype.contextmenu_command_handlers = new Object();
+rcube_webmail.prototype.contextmenu_disable_multi = new Array('#reply','#reply-all','#reply-list','#forward','#forward-attachment','#print','#edit','#viewsource','#download','#open','#edit');
 
 function rcm_contextmenu_update() {
 	if (!rcmail.env.flag_for_deletion && rcmail.env.trash_mailbox && rcmail.env.mailbox != rcmail.env.trash_mailbox)
@@ -319,7 +319,9 @@ function rcm_update_options(el) {
 			else
 				$('#rcmAddressMenu').enableContextMenuItems(rcmail.contextmenu_disable_multi.join(','));
 
-			if (rcmail.env.address_sources[rcmail.env.source].readonly)
+			var ab_src = rcmail.env.source ? rcmail.env.source : matches[1].split('-', 2)[1];
+
+			if (rcmail.env.address_sources[ab_src].readonly)
 				$('#rcmAddressMenu').disableContextMenuItems('#edit,#delete');
 			else
 				$('#rcmAddressMenu').enableContextMenuItems('#edit,#delete');
@@ -364,9 +366,11 @@ function rcm_addressmenu_init(row) {
 					case 'compose':
 					case 'delete':
 					case 'moveto':
+						var ab_src = rcmail.env.source ? rcmail.env.source : matches[1].split('-', 2)[1];
+
 						if (command == 'moveto') {
 							// check for valid taget
-							if (rcmail.env.rcm_destbook == rcmail.env.source || rcmail.env.contactfolders['G' + rcmail.env.rcm_destsource + rcmail.env.rcm_destgroup].id == rcmail.env.group)
+							if (rcmail.env.rcm_destbook == ab_src || (rcmail.env.rcm_destgroup && rcmail.env.contactfolders['G' + rcmail.env.rcm_destsource + rcmail.env.rcm_destgroup].id == rcmail.env.group))
 								return;
 						}
 
@@ -376,9 +380,6 @@ function rcm_addressmenu_init(row) {
 							if (!rcmail.contact_list.in_selection(rcmail.env.cid)) {
 								prev_sel = rcmail.contact_list.get_selection();
 								rcmail.contact_list.select(rcmail.env.cid);
-
-								if (!(command == 'moveto' && rcmail.env.rcm_destbook.substring(0, 1) == 'G') && command != 'compose')
-									rcmail.contact_list.remove_row(rcmail.env.cid, false);
 							}
 							else if (rcmail.contact_list.get_single_selection() == rcmail.env.cid) {
 								rcmail.env.cid = null;
@@ -389,8 +390,14 @@ function rcm_addressmenu_init(row) {
 							}
 						}
 
+						if (command == 'delete')
+							rcmail.env.cid = null;
+
 						rcmail.drag_active = true;
-						rcmail.command(command, rcmail.env.contactfolders['G' + rcmail.env.rcm_destsource + rcmail.env.rcm_destgroup], $(el));
+						if (rcmail.env.rcm_destgroup)
+							rcmail.command(command, rcmail.env.contactfolders['G' + rcmail.env.rcm_destsource + rcmail.env.rcm_destgroup], $(el));
+						else
+							rcmail.command(command, rcmail.env.contactfolders[rcmail.env.rcm_destsource], $(el));
 						rcmail.drag_active = false;
 
 						if (prev_sel) {
@@ -497,7 +504,17 @@ function rcm_groupmenu_update(action, props) {
 				.html('&nbsp;&nbsp;' + props.name);
 
 			var li = $('<li>').addClass('contactgroup').append(link);
-			$(li).insertAfter($('#rcm_contextaddr_' + props.source));
+
+			var sibling = $('#rcm_contextaddr_' + props.source);
+			$('a[id^="rcm_contextgrps_G"]').each(function(i, elem) {
+				if (props.name.toUpperCase().trim() >= $(this).text().toUpperCase().trim())
+					sibling = $(elem).parent();
+				else
+					return false;
+			});
+
+			$(li).insertAfter($(sibling));
+
 			rcm_groupmenu_init(props.li);
 			break;
 		case 'update':
@@ -518,6 +535,19 @@ function rcm_groupmenu_update(action, props) {
 				else {
 					$('#rcm_contextgrps_G' + gid).html('&nbsp;&nbsp;' + props.name);
 				}
+
+				row = $('#rcm_contextgrps_G' + gid).parent().clone(true);
+				$('#rcm_contextgrps_G' + gid).parent().remove();
+
+				var sibling = $('#rcm_contextaddr_' + props.source);
+				$('a[id^="rcm_contextgrps_G"]').each(function(i, elem) {
+					if (props.name.toUpperCase().trim() >= $(this).text().toUpperCase().trim())
+						sibling = $(elem).parent();
+					else
+						return false;
+				});
+
+				$(row).insertAfter($(sibling));
 			}
 
 			break;
@@ -530,25 +560,27 @@ function rcm_groupmenu_update(action, props) {
 }
 
 $(document).ready(function() {
-	// init message list menu
-	if ($('#rcmContextMenu').length > 0) {
-		rcmail.addEventListener('listupdate', function(props) { rcm_contextmenu_update(); } );
-		rcmail.addEventListener('insertrow', function(props) { rcm_contextmenu_init(props.row.id); } );
-	}
+	if (window.rcmail) {
+		// init message list menu
+		if ($('#rcmContextMenu').length > 0) {
+			rcmail.addEventListener('listupdate', function(props) { rcm_contextmenu_update(); } );
+			rcmail.addEventListener('insertrow', function(props) { rcm_contextmenu_init(props.row.id); } );
+		}
 
-	// init folder list menu
-	if ($('#rcmFolderMenu').length > 0)
-		rcmail.add_onload('rcm_foldermenu_init();');
+		// init folder list menu
+		if ($('#rcmFolderMenu').length > 0)
+			rcmail.add_onload('rcm_foldermenu_init();');
 
-	// init contact list menu
-	if ($('#rcmAddressMenu').length > 0)
-		rcmail.addEventListener('insertrow', function(props) { rcm_addressmenu_init(props.row.id); } );
+		// init contact list menu
+		if ($('#rcmAddressMenu').length > 0)
+			rcmail.addEventListener('insertrow', function(props) { rcm_addressmenu_init(props.row.id); } );
 
-	// init group list menu
-	if ($('#rcmGroupMenu').length > 0) {
-		rcmail.add_onload('rcm_groupmenu_init("#directorylistbox li");');
-		rcmail.addEventListener('group_insert', function(props) { rcm_groupmenu_update('insert', props); } );
-		rcmail.addEventListener('group_update', function(props) { rcm_groupmenu_update('update', props); } );
-		rcmail.addEventListener('group_delete', function(props) { rcm_groupmenu_update('remove', props); } );
+		// init group list menu
+		if ($('#rcmGroupMenu').length > 0) {
+			rcmail.add_onload('rcm_groupmenu_init("#directorylistbox li");');
+			rcmail.addEventListener('group_insert', function(props) { rcm_groupmenu_update('insert', props); } );
+			rcmail.addEventListener('group_update', function(props) { rcm_groupmenu_update('update', props); } );
+			rcmail.addEventListener('group_delete', function(props) { rcm_groupmenu_update('remove', props); } );
+		}
 	}
 });
\ No newline at end of file
